<?php

/** ********************************************************************************************* **/
/**                                                                                               **/
/** cipux.php                                                                                     **/
/**                                                                                               **/
/** Author : Jean-Charles Siegel, jeancharles.siegel@gmail.com, Benjamin Sonntag                  **/
/**                                                                                               **/
/** Year : 2006-03-19                                                                             **/
/**                                                                                               **/
/** Licence : GPLv2                                                                               **/
/**                                                                                               **/
/** Edited for : Debian Edu/Skolelinux, CipUX                                                     **/
/**                                                                                               **/
/** Description : This file defines php functions that call Cipux API using a XML RPC Client.     **/
/**               The configuration parameters are the defined below.                             **/
/**                                                                                               **/
/** ********************************************************************************************* **/

include("cipux_config.php");

/**
 * CIPUX XML RPC client library
 * Connect to a CIPUX XML RPC Server and send him authenticated requests.
 * Please see the wiki link for more information.
 * http://wiki.skolelinux.fr/cgi-bin/skolelinux/moin.cgi/XmlRpc
 */
class cipux_xml {
  
  var $server; /** XML RPC Server connection handle */
  var $faultCode = 0;  /** Last XML RPC Fault Code (0 = OK) */
  var $faultString = ""; /** Last XML RPC Fault Reason ("" = OK) */
  var $cookie = ""; /** The session cookie (usually from http cookies) */
  var $login_ticket;
  var $ticket;

  /** Constructor: initialize the xml rpc client library */
  function cipux_xml() {
    include_once("lib/xmlrpc.inc");
    $this->server = new xmlrpc_client(CIPUX_RPC_PATH, CIPUX_RPC_HOST, CIPUX_RPC_PORT, CIPUX_RPC_TRANSPORT);
    $this->server->setSSLVerifyPeer(false);
    $this->server->setCertificate("/etc/cipuxphp/rpc/cert/cipux_rpcd.pem","");
    $this->server->setDebug(0);
  }
  
  /** This method will affect the params to the class attributs
   * @param string $login_ticket
   * @param string $ticket
   */
  function set_ticket ($login_ticket, $ticket) {
    $this->login_ticket = $login_ticket;
    $this->ticket = $ticket;
  }
  
  /** Set the user cookie. Either automatically from a HTTP cookie 
   * from the remote user, Or manually with a function parameter.
   * @param string $cookie The cookie name (not required)
   */
  function set_cookie($cookie = "") {
    if (!$cookie) 
      $this->cookie = $_COOKIE[CIPUX_RPC_COOKIE_NAME];
    else
      $this->cookie = $cookie;
  }

  /** Those functions are XML RPC specific, 
   *  and manage the user authentication, ping, and logout.
   * @param string $username The user login
   * @param string $password The user password
   * @param boolean $sendcookie Should we send the session cookie through HTTP  ? (default yes)
   * @return string a session cookie or false if an error occured
   */
  function rpc_login ($username,$password,$sendcookie = true) {
    $f = new xmlrpcmsg('login', array(new xmlrpcval($username),new xmlrpcval($password)));
    $v = $this->_standard_call($f);
    /*if ($v) {
      set_cookie(CIPUX_RPC_COOKIE_NAME,$v,0); 
    }*/
    $this->ticket = $v;
    $this->login_ticket = $username;
    return $v;
  }

  /** Logout from the XML-RPC server */
  function rpc_logout() {
    $f = new xmlrpcmsg('cipux_rpc_logout', array(new xmlrpcval($cookie)));
    return $this->_standard_call($f);
  }
  
  /** Test if the server is on */
  function rpc_ping () {
    $f = new xmlrpcmsg('cipux_rpc_ping', array(new xmlrpcval($cookie)));
    return $this->_standard_call($f);
  }


  /** Make a simple sum between to arguments
   * @return int wich is the result of the sum
   */
  function sum($arg1, $arg2) {
    $f = new xmlrpcmsg('sum', array(new xmlrpcval($arg1), new xmlrpcval($arg2)));
    return $this->_standard_call($f);
  }
  

  /** List the user accounts and return an array with all login names
   * @return array an array of string containing usernames, or false if an error occured.
   */
  function list_users() {
    $f = new xmlrpcmsg('cipux_task_list_users', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket)));
    return $this->_standard_call($f);
  }


  /** List the teachers accounts and return an array with all login names
   * @return array an array of string containing usernames, or false if an error occured.
   */
  function list_teachers() {
    $f = new xmlrpcmsg('cipux_task_list_teachers', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket)));
    return $this->_standard_call($f);
  }
  

   /** List the students accounts and return an array with all login names
   * @return array an array of string containing usernames, or false if an error occured.
   */
  function list_students() {
    $f = new xmlrpcmsg('cipux_task_list_students', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket)));
    return $this->_standard_call($f);
  }
  
  /** List the guests accounts and return an array with all login names
   * @return array an array of string containing usernames, or false if an error occured.
   */
  function list_guests() {
    $f = new xmlrpcmsg('cipux_task_list_guests', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket)));
    return $this->_standard_call($f);
  }
  
  
  /** List the functions exported by the rpc server.
   * @return array an array of string containing function names, or false if an error occured.
   */
  function list_rpc_functions() {
    $f = new xmlrpcmsg('cipux_task_list_rpc_functions', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket)));
    return $this->_standard_call($f);
  }

  /** List the cipux roles
   * @return array an array of string containing function names, or false if an error occured.
   */
  function list_roles() {
    $f = new xmlrpcmsg('cipux_task_list_roles', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket)));
    return $this->_standard_call($f);
  }
  
    
  /** Create a new user account with the following attributes : 
   * @param string $userlogin The user login name  
   * @param string $username The user name
   * @param string $userlastname The user lestname
   * @param string $usertype The user type name
   * @param string $password1 The user password
   * @param string $password2 The user password confirmation
   * @return integer $uid Returns a user id or FALSE if an error occured.
   */
  function create_user($userlogin, $username, $userlastname, $usertype, $password) {
    $f = new xmlrpcmsg('cipux_task_create_user', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket),new xmlrpcval($userlogin), new xmlrpcval($username),new xmlrpcval($userlastname), new xmlrpcval($usertype),new xmlrpcval($password)));
    return $this->_standard_call($f);
  }
  
  /** Get firstname of a user
   * @param string $userlogin the login of the user
   * @return string/array
   */
  function tell_me_user_firstname ($userlogin) {
    $f = new xmlrpcmsg('cipux_task_tell_me_user_firstname', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket),new xmlrpcval($userlogin)));
    return $this->_standard_call($f);
  }
  
  /** Get lastname of a user
   * @param string $userlogin the login of the user
   * @return string/array
   */
  function tell_me_user_lastname ($userlogin) {
    $f = new xmlrpcmsg('cipux_task_tell_me_user_lastname', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket),new xmlrpcval($userlogin)));
    return $this->_standard_call($f);
  }

  /** Get firstname and lastname of a user
   * @param string $userlogin the login of the user
   * @return string/array
   */
  function tell_me_user_canonical_name ($userlogin) {
    $f = new xmlrpcmsg('cipux_task_tell_me_user_canonical_name', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), new xmlrpcval($userlogin)));
    return $this->_standard_call($f);
  }

    /** Get role of a user
   * @param string $userlogin the login of the user
   * @return string/array
   */
  function tell_me_user_role ($userlogin) {
    $f = new xmlrpcmsg('cipux_task_tell_me_user_role', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), new xmlrpcval($userlogin)));
    return $this->_standard_call($f);
  }
  
  function enable_user() {
  }
  

  function enable_user_posix_account() {
  }
  
  /** Change the password of a user
   * @param srting $username the group name
   * @param srting $password the new password of the user
   * @return ?
   */
  function change_user_password($username, $password) {
    $f = new xmlrpcmsg('cipux_task_change_user_password', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket),new xmlrpcval($username),new xmlrpcval($password)));
    return $this->_standard_call($f);
  }

  /** Change the role of a user
   * @param srting $username the group name
   * @param srting $role the new role of the user
   * @return ?
   */
  function change_user_role($username, $role) {
    $f = new xmlrpcmsg('cipux_task_change_user_role', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket),new xmlrpcval($username),new xmlrpcval($role)));
    return $this->_standard_call($f);
  }  
  
  /**Delete a user from LDAP
   * @param string $userlogin the user login
   * @return ?
   */
  function delete_user($userlogin) {
    $f = new xmlrpcmsg('cipux_task_delete_user', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), new xmlrpcval($userlogin)));
    return $this->_standard_call($f); 
  }
  

  function delete_user_homedir() {
  }
  

  function disable_user() {
  }
  

  function disable_user_posix_account() {
  }
  
  
  /** Add a new user in a group
   * @param string $groupname The group name
   * @param string $username The user name
   * @return ?
   */  
  function add_user_to_course($groupname, $username) {
    $f = new xmlrpcmsg('cipux_task_add_user_to_course', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), new xmlrpcval($groupname),new xmlrpcval($username)));
    return $this->_standard_call($f);
  }
  
  /** Create a new group, with the following attributes
   * @param string $groupname The group name
   * @param string $password The group password
   * @return integer $uid Returns a group id or FALSE if an error occured.
   */
  function create_course($groupname, $password) {
    $f = new xmlrpcmsg('cipux_task_create_course', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), new xmlrpcval($groupname),new xmlrpcval($password)));
    return $this->_standard_call($f);
  }
  

   /** List the courses and return an array with all courses' names
   * @param string $uid the uid of the user
   * @param string $this->ticket the ticket given by the login function
   * @return array an array of string containing courses' name, or false if an error occured.
   */
  function list_courses() {
    $f = new xmlrpcmsg('cipux_task_list_courses', array(new xmlrpcval($this->login_ticket),new xmlrpcval($this->ticket)));
    return $this->_standard_call($f);
  }
  
  
  function disable_course() {
  }
  

  function enable_course() {
  }
  

  function enable_course_posix_account() {
  }
  

  function disable_course_posix_account() {
  }
  

  /** Change the password of a group
   * @param srting $groupname the group name
   * @param srting $password the new pasword of the group
   * @return ?
   */
  function change_course_password($groupname, $password) {
    $f = new xmlrpcmsg('cipux_task_change_course_password', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), new xmlrpcval($groupname),new xmlrpcval($password)));
    return $this->_standard_call($f);
  }
  
  
   /** Delete the course using the param
   * @param srting $groupname the group name
   * @return array an array of string containing courses' name, or false if an error occured.
   */
  function delete_course($groupname) {
    $f = new xmlrpcmsg('cipux_task_delete_course', array(new xmlrpcval(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), $groupname)));
    return $this->_standard_call($f);  
  }

  
  /** List the members of a course using the param
   * @param string $groupname the group name
   * @return array an array of string or false if an error occured.
   */
  function list_user_members_of_course($groupname) {
    $f = new xmlrpcmsg('cipux_task_list_user_members_of_course', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), new xmlrpcval($groupname)));
    return $this->_standard_call($f);  
  }
     
   
  /**Remove a new user from a group
   * @param string $groupname The group name
   * @param string $username The user name
   * @return ?
   */
  function remove_user_from_course($groupname, $username) {
    $f = new xmlrpcmsg('cipux_task_remove_user_from_course', array(new xmlrpcval($this->login_ticket), new xmlrpcval($this->ticket), new xmlrpcval($groupname),new xmlrpcval($username)));
    return $this->_standard_call($f);
  }
  
  /** List the first letters of each users
   * @return array an array of caracters
   */
  function list_users_first_letter () {
    $result = $this->list_users();
    $return = Array();
    $j = 0;
    for ($i=0 ; $i<sizeof($result) ; $i++) {
      if (!in_array($result[$i][0], $return)) {
        $return[$j] = $result[$i][0];
	$j++;
      }
    }
    sort($return);
    return $return ;
  }
  
  /** Private function that do standard XML RPC call
   * @private
   * 
   */
  function _standard_call($f) {
    $r = $this->server->send($f,0, CIPUX_RPC_TRANSPORT);
    $v = $r->value();
    $this->faultCode = $r->faultCode();
    $this->faultString = $r->faultString();
    
    if(!$this->faultCode) {
      return php_xmlrpc_decode($v);
    } else {
      return false;
    }
  }
}
?>
