#!/usr/bin/perl -w
# *========================================================================*
# || cipux_change                                                         ||
# || This script changes a CipUX LDAP entry from the server. It uses then ||
# || the change.conf to change the user from the system and to change the ||
# || home directory.                                                      ||
# ||                                                                      ||
# || Copyright: (c) 2005-2006  by Christian Kuelker all Rights reserved!  ||
# || Licence:   GPLv2 or later                                            ||
# ||                                                                      ||
# || Revision:  2006-11-03                                                ||
# ||                                                                      ||
# *========================================================================*

=pod

=head1 NAME

cipux_change - Version <[CIPUX::VERSION]>  2006-11-03 CipUX

=head1 SYNOPSIS

	cipux_change [OPTIONS]

	Options: (one nessessary)
        (01)  -u <RDN> -e <RDN>     :changes a user node dn
        (02)  -c <RDN> -e <RDN>     :changes a course node dn
        (03)  -m <RDN> -e <RDN>     :changes a machine node dn
        (04)  -n <RDN> -e <RDN>     :changes a netgroup dn
        (05)  -r <RDN> -e <RDN>     :changes a room node dn
        (06)  -t <RDN> -e <RDN>     :changes a hardware type node dn
        (07)  -i <RDN> -e <RDN>     :changes a image node dn
        (08)  -s <RDN> -e <RDN>     :changes a image slot node dn
        (09)  -x <RDN> -e <RDN>     :changes a configuration node dn
        Other options:
        (10)  -h            :get help message

=head1 ABSTRACT

This script changes a CipUX node dn (LDAP entry) from the server.
Possible nodes are:

  User (-u),
  Course (-c),
  Machine (-m),
  Negtgroup (-n),
  Room (-r),
  Hardware Type (-t),
  Image (-i),
  Image Slot (-s),
  Configuration (-x)

=head1 REQUIREMENTS

  CipUX::CiBot
  ldapmodrdn
  Getopt::Std

=cut

package cipux_change;

# +========================================================================+
my $PART=                    "Global Part";#                               |
# +========================================================================+
my $BRAND   = "cipworx";
my $PROJECT = "cipux";
my $SCRIPT  = "cipux_change";
my $VERSION = "<[CIPUX::VERSION]>";
my $BUILD   = "<[CIBOT::BUILD]>";

&log($SCRIPT,"="x76);

my @mandatory = qw(u c m n r t i s x);

# +========================================================================+
# |                           Module                                       |
# +========================================================================+

# Load all modules
use strict;
use CipUX::CiBot;
use Getopt::Std;

# +========================================================================+
$PART=                       "Environment";#                               |
# +========================================================================+

# use of environment module
my %opt=();
my $opt = join(':',@mandatory);
getopts($opt."e:h",\%opt);

# test for help
if(defined($opt{h})){usage();exit(0)}

# test for e
if(not defined($opt{e})){
  usage();
  # error::0206=(change.pl) Need mandatory option -e, please give it to me.
  &error('0206');
  exit(1);
}

# test for a second option and remember it in $option
my $option = '';
foreach(keys %opt){
  if($_ ne 'e'){
    if(defined($opt{$_})){
      $option=$_;
      &log($SCRIPT,"+ option is [$option]");
    }
  }
}
if($option eq ''){
  usage();
  # error::0208=(change.pl) Need one not empty mandatory option from:
  my $mandatory = join(', ',@mandatory);
  &error('0208',$mandatory);
  exit(1);
}

# test if 2nd option is a mandatory option
if($option eq 'u'
   or $option eq 'c'
   or $option eq 'm' 
   or $option eq 'n'
   or $option eq 'r' 
   or $option eq 't' 
   or $option eq 'i' 
   or $option eq 's' 
   or $option eq 'x'
  ){
  &log($SCRIPT,"- got mandatory option [$option]");
}else{
  &log($SCRIPT,"- NO mandatory option! Got fake option [$option]");
  usage();
  # error::0207=(change.pl) Need one mandatory option from:
  my $mandatory = join(', ',@mandatory);
  &error('0207',$mandatory);
  exit(1);
}

my $rdn1 = $opt{$option};
&log($SCRIPT,"- got rdn1 [$rdn1]");

my $rdn2 = $opt{e};
&log($SCRIPT,"- got rdn2 [$rdn2]");

my $s1  = "Ldap_DN_".uc($option)."_Prefix";
my $s2  = "Ldap_DN_".uc($option);
my $dn  = $cipux{$s1}."=".$rdn1.",".$cipux{$s2};
my $rdn = $cipux{$s1}."=".$rdn2;

&log($SCRIPT,"- got Ldap_DN_?_Prefix [$s1]");
&log($SCRIPT,"- got Ldap_DN_?        [$s2]");
&log($SCRIPT,"- got dn  [$dn]");
&log($SCRIPT,"- got rdn [$rdn]");

# +========================================================================+
$PART=                       "Check if Node exist";#                     |
# +========================================================================+

#  $node_href = &cibot_list('U','uid');
my $node_href = &cibot_list(uc($option),$s1);

# test if exists
if(defined($$node_href{$rdn1})){
  &log($SCRIPT,"- node [$rdn1] exist, it meight be changed");
}else{
  &log($SCRIPT,"- node is nonexistend, it meight not be changed");
  &error('0202',$rdn1);# error::0202=(cipux_change) The object does not exist:
  exit(1);
}

# +========================================================================+
$PART=                       "Main";#                                      |
# +========================================================================+

&log($SCRIPT,"* try to change dn [$dn] to rdn [$rdn]");

# [/usr/bin/ldapmodrdn -r -D 'cn=admin,ou=People,dc=skole,dc=skolelinux,dc=no' -p 389 -h localhost -x -ZZ -y /etc/cipux/ldappassword.conf 'cn=bibeutlinas,ou=Group,dc=skole,dc=skolelinux,dc=no' 'cn=bibeutlin'

my $cmd = "$cipux{Ldap_Modrdn_Command} -r -D '$cipux{Ldap_Root_DN}' -p $cipux{Ldap_Port} -h $cipux{Ldap_Host} -x $cipux{Ldap_Start_TLS} -y /etc/cipux/ldappassword.conf '$dn' '$rdn'";

# disabled for security reason
#&log($SCRIPT,"- cmd: [$cmd]");

my $pid = open(CMD,"$cmd|")
  # error::0205=(change.pl) Can't execute ldap change command:
  or error('0205',$cmd);
my $output = <CMD>;
my $ok = close(CMD);
# error::0204=(change.pl) Error while executing ldap change command:
&error('0204',$cmd) if !defined($pid);
# error::0203=(change.pl) Executing ldap change command not successful:
&error('0203',$cmd) if !$ok;
# TODO different exit values!
#      0 = ok
#      1 = LDAP not ok
#      2 = LDAP ok, but node not found 'No such node (32)'
#      3 = LDAP ok, but change.conf CMDs error

exit(0);

# +========================================================================+
$PART=              "Subroutines";#                                        |
# +========================================================================+

=pod

=head1 FUNCTIONS

=cut


# .........................................................................
# :                               usage                                   :
# .........................................................................

=pod

=head2 usage

Prints usage message.

=cut

sub usage{
  &log($SCRIPT,"* print help message");	

  print "-"x76;
  print "\n";
  print "USAGE: ($BRAND-$PROJECT-$SCRIPT-$VERSION)\n\n";
  print "       $SCRIPT [OPTIONS]\n\n";
  print "Changes a node dn from CipUX!";
  print "\n";
  print "Options: (need only one set)\n";
  print "(01)  -u <RDN> -e <RDN>     :changes a user dn\n";
  print "(02)  -c <RDN> -e <RDN>     :changes a course dn\n";
  print "(03)  -m <RDN> -e <RDN>     :changes a machine dn\n";
  print "(04)  -n <RDN> -e <RDN>     :changes a netgroup dn\n";
  print "(05)  -r <RDN> -e <RDN>     :changes a room dn\n";
  print "(06)  -t <RDN> -e <RDN>     :changes a hardware type dn\n";
  print "(07)  -i <RDN> -e <RDN>     :changes a image dn\n";
  print "(08)  -s <RDN> -e <RDN>     :changes a image slot dn\n";
  print "(09)  -x <RDN> -e <RDN>     :changes a configuration dn\n";
  print "Other Options:\n";
  print "(10)  -h            :this message\n";
  print "\n";
  print "-"x76;
  print "\n";
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================


=head1 SEE ALSO

See the CipUX webpage and the manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2006 by Christian Kuelker 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
MA 02110-1301, USA.

=cut

