#!/usr/bin/perl -w

use POSIX;
use strict;
use IO::Socket;
use File::Basename;
use Frontier::Client;


my $basename = basename ($0);
my $pidfile="/var/run/$basename.pid";
my $ipt_cmd = "/sbin/iptables";
my $DEBUG = 1;
my @noout_reject = ("-p tcp --destination-port 8080", "-p tcp --destination-port 3128");
my @noout_accept = ("-d  10.0.0.0/8", "-d  192.168.0.0/16",
	"-d 172.16.0.0/12", "-d 169.254.0.0/16", "-o lo");
my $remote_host = "tjener";

my $login = "cipadmin";
my $https_url = "https://$remote_host:8000/RPC2";


=pod

=head1 NAME

cipux_client - Version <[CIPUX::VERSION]>  2006-07-26

=head1 SYNOPSIS

        cipux_client

=head1 ABSTRACT

This is the CipUX Client for the CipUX Notifier. It creates the
netfilter chain "noout" at first and then connects to the Notifier
and changes the iptables rules if necessary to allow or disallow users
the access to the internet (according to the LDAP-Values).

=head1 REQUIREMENTS

 iptables, Kernel support for Netfilter, libipt_owner.so,
 libfrontier-rpc-perl libcrypt-ssleay-perl

=head1 DESCRIPTION

Start the CipUX Client after the CipUX Notifier.

=head1 USAGE

 /usr/sbin/cipux_client

=cut

# the reject calls are executed first

if (open (PID, "<$pidfile")) {
  my $oldpid = <PID>;
  chomp $oldpid;
  die "$basename ERROR 0000: already running" if (kill 0, $oldpid);
  close(PID);
}

#if (open (PASSWORD, "/etc/cipux
my $password = 'pw';

debug ("started $basename");

my $pid = fork;
exit if $pid;
die "cipux-client ERROR 0001: Could not fork: $!" unless defined($pid);

# detach from terminal
POSIX::setsid()
  or die "cipux-client ERROR 0002: Cannot start a new session: $!";

open(PID, ">$pidfile")
  or die "cipux-client ERROR 0003: Cannot write PID $$ to file $pidfile";
print PID $$;
close(PID);


$SIG{TERM} = sub { unlink $pidfile; die "$basename terminated"; };

my ($user, %ldap, %iptables, @ldap, @iptables);

my $server = Frontier::Client->new(url => $https_url);

if ($server->call('ping') ne "OK") {
  die "$basename ERROR : Could not connect to xmlrpc server";
}


my $ticket = $server->call('login', $login, $password);
if(defined($ticket)) {
  if ($ticket eq '') {
    die "wrong login or password for the xmlrpc server!";
  }
} else {
  die "wrong login or password for the xmlrpc server!";
}


my $sock = new IO::Socket::INET (
                                 PeerAddr => $remote_host,
                                 PeerPort => '7676',
                                 Proto => 'tcp',
                                );
die "Could not create socket: $!" unless $sock;

qx($ipt_cmd -L noout) or init_iptables();

while (<$sock>) {
	debug ("timestamp: $_");

	my $user_ref = $server->call('cipux_task_tell_me_users_internet_status',$login, $ticket);
	for (@$user_ref){
	  chomp;
	  my ($user, $state) = split /\t/;
	  $ldap{$user}="r" if ($state eq "reject");
	}
#	@ldap = qx(cipux_get_value -U  -e cipuxInternetStatus|grep reject|cut -f 1);
#	foreach $user (@ldap) {
#		chomp $user;
#		$ldap{$user}="r";
#		debug ("ldap user: #$user#");
#	}
	
	@iptables=qx($ipt_cmd -L OUTPUT |grep ^noout);
	for (@iptables) {
		my $user=((split (/OWNER UID match /))[1]);
		$user =~ s/\s*$//;
		$iptables{$user}="r";
		debug ("iptables user: #$user#");
	}
	
	for (keys %ldap) {
		if (not defined $iptables{$_})
		{
			qx($ipt_cmd -I OUTPUT -m owner --uid-owner $_ -j noout);
			debug ("$ipt_cmd -I OUTPUT -m owner --uid-owner $_ -j noout");
		}
	}
	
	for (keys %iptables) {
		if (not defined $ldap{$_})
		{
			qx($ipt_cmd -D OUTPUT -m owner --uid-owner $_ -j noout);
			debug ("$ipt_cmd -D OUTPUT -m owner --uid-owner $_ -j noout");
		}
	%iptables=%ldap=();
	}
}
close($sock);
unlink ($pidfile);

sub init_iptables {
 qx($ipt_cmd -N noout);
 for (@noout_reject) {
  qx($ipt_cmd -A noout $_ -j REJECT);
 }
 for (@noout_accept) {
  qx($ipt_cmd -A noout $_ -j ACCEPT);
 }
 qx($ipt_cmd -A noout  -j REJECT);
 die "Error when trying to set iptables rules: $!" if ($! ne "");
}

sub debug {
  if($DEBUG){
    my $date = `date`;
    my $msg = shift;
    chomp($date);
    open(LOG,">>/var/log/cipux/$basename.log");
    print LOG "cipuxxmlrpcd ($date): $msg\n";
    close(LOG);
  }
}


=head1 SEE ALSO

See the CipUX webpage and the manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Benedikt Wildenhain E<lt>benedikt.wildenhain@esperanto.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006 by Benedikt Wildenhain

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
MA 02110-1301, USA.

=cut
