#!/usr/bin/perl -w

use strict;
use IO::Socket;
use threads;
use threads::shared;
use POSIX;
use Config;
use File::Basename;
use Frontier::Client;


my $basename = basename ($0);
my $login = "cipadmin";
my $http_url = "https://localhost:8000/RPC2";
my $pidfile="/var/run/$basename.pid";
my $DEBUG = 1;

my $password = 'pw';

=pod

=head1 NAME

cipux_notifier - Version <[CIPUX::VERSION]>  2006-07-26

=head1 SYNOPSIS

        cipux_notifier

=head1 ABSTRACT

This is the CipUX Notifier to be used by the CipUX Clients. It
monitors the LDAP configuration value CipUX_Notify_Clients and notifies
all connected Clients when a change occurs.

=head1 REQUIREMENTS

 cipux_get_value

=head1 DESCRIPTION

Start the CipUX Notifier before the CipUX Clients.

=head1 USAGE

 /usr/sbin/cipux_notifier

=cut


$Config{useithreads} or die "Recompile Perl with threads to run this program.";

if (open (PID, "<$pidfile")) {
  my $oldpid = <PID>;
  if (undef $oldpid) {
   chomp $oldpid;
   die "$basename ERROR 0000: already running" if (kill 0, $oldpid);
   close(PID);
  }
}

debug ("started $basename");

my $pid = fork;
exit if $pid;
die "$basename ERROR 0001: Could not fork: $!" unless defined($pid);

# detach from terminal
POSIX::setsid()
  or die "$basename ERROR 0002: Can't start a new session: $!";

open(PID,">$pidfile")
  or die "$basename ERROR 0003: Can't write PID $$ to file $pidfile";
print PID $$;
close(PID);

$SIG{TERM} = sub { unpid(); };


my $server = Frontier::Client->new(url => $http_url);

if ($server->call('ping') ne "OK") {
  die "$basename ERROR : Could not connect to xmlrpc server";
}


my $ticket = $server->call('login', $login, $password);
if(defined($ticket)) {
  if ($ticket eq '') {
    die "wrong login or password for the xmlrpc server!";
  }
} else {
  die "wrong login or password for the xmlrpc server!";
}


my $clientssock = new IO::Socket::INET (
#                                 LocalHost => '',
                                 LocalPort => '7676',
                                 Proto => 'tcp',
                                 Listen => SOMAXCONN,
                                 ReuseAddr => 1,
                                );
die "Could not create client socket: $!" unless $clientssock;

my $i=0;

my $seq : shared=0;

my $thread = threads -> new(sub {
  while (sleep 3) {
   $seq = qx(cipux_task_tell_me_cipux_config_variable_value CipUX CipUX_Client_Notify);
   #$seq = ($server -> call('cipux_task_tell_me_cipux_config_variable_value', $login, $ticket, 'CipUX', 'CipUX_Client_Notify')) -> [0];
   debug (__LINE__.$seq);
   #debug (warn $seq -> [0]);
  }
});

while (my $client = $clientssock -> accept()) {
 my $thread = threads -> new(\&client_thread, $client, $i++);
}

unpid();

sub client_thread {
 my $client = shift;
 my $client_nr = shift;
 my $oldseq = $seq."-changed";
 while (sleep 1 and $client -> connected()) {
  if ($seq ne $oldseq) {
   print $client "$seq\n";
  }
  $oldseq = $seq;
 }
 close($client);
}


sub debug {
  if($DEBUG){
    my $date = `date`;
    my $msg = shift;
    chomp($date);
    open(LOG,">>/var/log/cipux/$basename.log");
    print LOG "cipuxxmlrpcd ($date): $msg\n";
    close(LOG);
  }
}

sub unpid {
  open(PID,">$pidfile")
    or die "$basename ERROR 0003: Can't write PID -1 to file $pidfile";
  print PID -1;
  close(PID);
}

=head1 SEE ALSO

See the CipUX webpage and the manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Benedikt Wildenhain E<lt>benedikt.wildenhain@esperanto.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006 by Benedikt Wildenhain

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
