#!/usr/bin/perl -w
# +=======================================================================+
# || /usr/sbin/cipux_rpcd                                                ||
# ||                                                                     ||
# || Daemon for the CipUX::Task API of CipUX                             ||
# || - XML-RPC Daemon                                                    ||
# || - local communication over socket                                   ||
# || - remote communication over stunnel and SSL                         ||
# || - test functions without authorization                              ||
# || - login function                                                    ||
# || - logout function                                                   ||
# || - session function                                                  ||
# || - debug function                                                    ||
# || - config file                                                       ||
# || - ticket with life span                                             ||
# || - ACL layer based on role for (self:all) on function level          ||
# ||                                                                     ||
# || (- trigger the cipux_bot_logon_ldap ?)                              ||
# ||                                                                     ||
# ||                                                                     ||
# ||                                                                     ||
# || (c) Copyright 2006 by Christian Kuelker, All rights reserved!       ||
# ||                                                                     ||
# || License: GPLv2 or later                                             ||
# ||                                                                     ||
# +=======================================================================+

# libfrontier-rpc-perl (debian package)

#use strict;
use warnings;
use Frontier::Daemon;
use CipUX::Task;
use CipUX::CiBot;
use Frontier::RPC2;
use Digest::MD5 qw(md5_hex);
use Authen::Simple::PAM;
use Authen::Simple::Password;

=pod

=head1 NAME

cipux_rpcd - Version <[CIPUX::VERSION]>  2006-07-02 CipUX

=head1 SYNOPSIS

        cipux_rpcd

=head1 ABSTRACT

This is the CipUX XML-RPC daemon for the CipUX::Task API. The 
daemon provides CipUX commands to RPC clients.

=head1 REQUIREMENTS

       CipUX::Task 
       POSIX
       Frontier::Daemon
       Frontier::RPC2
       Digest::MD5

=head1 DESCRIPTION

 (1) get a list @commands of subroutines from the CipUX::Task module
     @commans = (
     cipux_task_list_users
     cipux_task_list_roles
     ...
     )
 (2) create a hash with names of the commands as keys and local subroutine
     names as reference like notdirect_cipux_task_list_roles, where the left
     part is "notdirect_" and the right part is the subroutine name from CipUX::Task
     \%commands = ( 'cipux_task_list_roles' => \&notdirect_cipux_task_list_roles, )
 (3) execute the RPC server with this \%commands as methodes => \%commands
 (4) if the function main::notdirect_cipux_task_list_roles is called, it is not
     defined, so the sub AUTOLOAD subroutine will handle that.
     The reason why we can create main::cipux_task_list_role because 
     cipux_task_list_role is imported from CipUX::Task and is valid for
     main::cipux_task_list_role and CipUX::Task::cipux_task_list_role!
 (5) if the function (CipuX::Task::cipux_task_list_roles) is defined in
     %taskcommands, then the command as a representative function in CipUX::Task 
     and can be executed.

=head1 USAGE

 (1) send the test function 'ping' to the server.
     you will get an OK if the server is online.

 (2) send the loginfunction to the server
     ticket = login(uid,password)
     If the password is valid, you will get a ticket back.

 (3) The ticket is valid for 20 minutes.

 (4) You must use uid and ticket as first and second 
     parameter for EVERY cipux_task_* function call

 (5) you must use the "session" function to prevent idle timeout

 (6) use the "logout" function

=cut

# if there is config file
if(-e "/etc/cipux/rpc/rpc.conf"){
  &debug("- got config file");
}

&cibot_get_cipux_config("/etc/cipux/rpc/cipux_rpc.conf");

my $DEBUG = $cipux{CipUX_XML_RPC_Debug};

my $PORT  = $cipux{CipUX_XML_RPC_Port};

# do not change this value, or you open an unencryptet server to the world!
# if you ant to change the port of the outside server do this in 
# /etc/cipux/rpc/stunnel.conf
#my $ADDR  = "127.0.0.1";
my $ADDR  = "localhost";

# in minutes:
my $TTL   = $cipux{CipUX_XML_RPC_Ticket_TTL};

# to use plain LDAP auth
my $GET = "/usr/lib/cipux/lal/cipux_get_value";

# Calculating secret random seed for this session
# "S ISp&FtR0z$EU!We8DvpUzC26D0RE1pVW8vSXp9at5RUwXkWesmQvJY!w!LrLHdo^wB7f6lr7U9PGPTYhxTI!PhKjXhMmZZKckIi^Qbl&g^$Qir!9S5LIoo!J1bX*OHVw"
# I would like to call Math::TrulyRandom or Math::Random but 
# this packages seems not to available in Debian
srand();
my @chars = ("A" .. "Z", "a" .. "z", 0 .. 9, qw(! @ $ % ^ & *));
my $seed = join("",@chars[map{rand@chars}(1 .. 128)]); 

&debug("* starting server soon with ip = [$ADDR] on port = [$PORT]");
sleep(3); # if we are killed, we a have to sleep a while to regenerate ... ;-)

# add every CipUX::Task comand to the aviable commands
my @commands = &cipux_task_list_rpc_functions();
my %commands = ();
my %taskcommands = ();
my %time = ();

foreach (@commands){
  &debug("  - add command [$_]");
  my $ref = "notdirect_".$_;
  $commands{$_} = \&$ref;
  $taskcommands{"CipUX::Task::$_"} = "CipUX::Task::$_";
}
$commands{sum}     = \&sum;
$commands{ping}    = \&ping;
$commands{login}   = \&login;
$commands{logout}  = \&logout;
$commands{session} = \&session;
$commands{ttl}     = \&ttl;
#$commands{cipux_task_list_roles} = \&main::cipux_task_list_roles;

# TODO:
# chroot("/var/daemon")
#    or die "Couldn't chroot to /var/daemon: $!";

# Fork once, and let the parent exit.
&debug("* try to fork from $$ ...");
my $pid = fork;
exit if $pid;
die "cipux-xmlrpc ERROR 0002: Could not fork: $!" unless defined($pid);
&debug("  ... successfully forked to pid $pid -> $$");

# Dissociate from the controlling terminal that started us and stop being part 
# of whatever process group we had been a member of.
&debug("* try to escape from terminal ...");
use POSIX;
POSIX::setsid()
  or die "cipux-xmlrpc ERROR 0003: Can't start a new ession: $!";
&debug("  ..escape from terminal successful!");

# Trap fatal signals, setting a flag to indicate we need to gracefully exit.
&debug("* try to install new sigal handler ...");
my $time_to_die = 0;

sub signal_handler {
  $time_to_die = 1;
}

$SIG{INT} = $SIG{TERM} = $SIG{HUP} = \&signal_handler;
# trap or ignore $SIG{PIPE}
&debug("  ... successfully installed new signal handler");


# write the pid
&debug("* try to write pid to /var/run/cipux_rpcd.pid");
open(PID,">/var/run/cipux_rpcd.pid") 
  or die "cipux-xmlrpc ERROR 0004: Can't write PID $$ to file /var/run/cipux_rpcd.pid\n";
print PID $$;
close(PID);
&debug("  ... successfully wrote pid to /var/run/cipux_rpcd.pid");


# Here the code:
my $z = 0;
&debug("* try to bind to LocalAddr => $ADDR LocalPort => $PORT");
my $daemon =0;
until ($time_to_die) {
  sleep(3);
  &debug("RUNNING $z ... (if this value counts up continious, the server did not start correctly)");
   $z++;
  # $PORT var can not be assignet in new(), why? So leave it hard coded ...
   $daemon = Frontier::Daemon->new(
				LocalPort => "https(8001)",
                                LocalAddr => "127.0.0.1:8001",
                                ReuseAddr => 1,
                                Proto     => 'tcp',
				methods   => \%commands
			       );
# not implemented for Debian sarge
#                                ReusePort => 1,
   &debug("  - [$daemon]");
}

# +=======================================================================+
# || functions                                                           ||
# +=======================================================================+

=pod

=head1 PUBLIC FUNCTIONS

All CipUX::Task funtions are public. And this additional functions are
public too.

There are two kinds of public functions:

(1) Funtions without authorization

 - login
 - ping
 - sum

(2) Every other function is available only after using 'login'
function, with a uid as first paramater and a valid ticket as second
parameter.

Examples (pseudo code):

  - (reference to user list) = cipux_task_list_users( uid, ticket );
  - (true|false) = logout( uid, ticket );
  - (new ticket|false) = session( uid, ticket );

If the uid do not match, or the uid has not the authorisation to use the function,
or the group of the uid has not the authorisatzion to use the function, or the ticket is expired, or the ticket is not valid the request will not be fullfilled.

In other words: if the uid match and has the right and the role also has the right and the ticket is valid and is not expired, the request will be executed.

To see real examples have a look at cipux_rpc sample client.

=cut

sub AUTOLOAD {

  &debug("# sub 'AUTOLOAD' [$$]");
  my $uid = shift;
  &debug("  - got uid [$uid]");

  # check if we called right with uid
  if(defined($uid)){

    # check, if the uid was authenticated before
    if(defined($time{$uid})){
      &debug("  - stored time [$time{$uid}] for uid [$uid]");
      my $ticket = shift;
      #&debug("  - ticket [$ticket]");
      &debug("  - ticket [TICKET]");
      # check, if the uid is authenticated and has a valid not expired ticket
      #&debug("call => check_ticket($uid,$ticket,$time{$uid})");
      my $status = &check_ticket($uid,$ticket,$time{$uid});

      # if ticket status OK, user is authenticated
      if($status eq 'OK'){ 
	&debug("  ! uid is authenticated");
	use vars qw($AUTOLOAD);
	my $function = $AUTOLOAD;
	&debug("  - autload function name [$function]");
	# authorization (get role and ACLs)
        &cibot_get_ldap_config();

	# get the role to test ACL
	my $role = cipux_task_tell_me_user_role('rpc',$uid);
	&debug("  - uid [$uid] has role [$role]");

        # if we get a defined role
        if(defined($role)){
	  $function =~ s/^main::notdirect_/CipUX::Task::/g;
	  &debug("  - changed name to [$function]");

	  # check, if function is registerd
	  if(defined($taskcommands{$function})){# if function exists
	    &debug("    - function [$function] is registerd as RPC task");

	    if(defined($ldap{RPC}{$function})){# defined ACL rule
	      &debug("  - ACL rule for function $function defined");

	      # try to find out if the ACL rule and the role match
	      my $acl= $ldap{RPC}{$function};
	      my($acl_type,$authorization) = &check_acl($acl,$role); 
              my $return_ref = '' ;
              if($authorization){# authorization
		&debug("  - authorize uid [$uid] as role [$role] by acl [$acl]");
		$return_ref = &exec_function($uid,$acl_type,$function,@_);
		&debug("* ending auto loading ...");
		return [@$return_ref];
	      }else{# authorization
		&debug("    - uid [$uid] is not authorized to use [$function] as [$role]");
		# write a warning or not?
		#my $c = Frontier::RPC2->new;
		#my $xml = $c->encode_fault("99"," uid [$uid] is not authorized to use [$function] as [$role]");
		#&debug("XML [$xml]");
		#return $xml;
	      }
	
	    }else{# defined ACL rule
	      &debug("WARN: ACL rule for [$function] is not defined!");
	    }
	  }else{ # check, if function is registerd
	    &debug("WARN: task [$function] is not registered as RPC task"); 
	  }
	}else{#if we get a defined role
	  &debug("WARN: role of [$uid] isn't defined, you must assign a role");
	}
      }else{# status check# if ticket status OK, user is authenticated
	&debug("WARN: uid has no valid ticket, user not authenticated!");
      }
    }else{# check, if the uid was authenticated before
      &debug("WARN: uid was never authenticated! (in this session)");
    }
  }else{# check if we called right with uid
    &debug("WARN: false use of function. The first parameter must be uid!");
  }
  &debug("* ending auto loading ...");
  return undef;
}
	   


sub exec_function {
  my $uid = shift;
  my $acl_type = shift;
  my $function = shift;
  my @a = ();
  #foreach (@_){
  #  &debug("      - got argument: [$_]");
  #}
  if($acl_type eq "all"){
    @a = $taskcommands{$function}->('rpc',@_); # execute the function
  }elsif($acl_type eq "self"){
    my $untrusted = shift(@_); # delete untrusted uid (or whatever)
    unshift(@_,$uid); # add trusted uid
    if($untrusted ne $uid){
      &debug("WARN: sombody tries to fool the RPC server ($untrusted ne $uid)");
      &debug("WARN: or tries to execute a function with in the ACL self scope, which may not be suited for execution under self scope. Only functions fith the first parameter = UID (mayby also MID, CID) can be accessed be self scope. Check the ACL rights! Must be somthing like ...;self:none");
    }else{
      # functions wit ACL "self eq not none" must have an ID 
      # as the first parameter!
      @a = $taskcommands{$function}->('rpc',@_); # execute the function
    }
  }
  #foreach (@a){
  #  &debug("      - got return value: [$_]");
  #}
  return \@a;
}

sub check_acl {
  my $acl = shift;
  my $role = shift;
  my $acl_type = '';
  my $authorization = 0;
  # functionname=all:admin assistant;self:student 
  &debug("    - got ACL [$acl]");
  &debug("    - got role [$role]");
  my $acl_all = $acl;
  $acl_all =~ s/^all:(.*);.*$/$1/g;
  &debug("    - calculate all ACL [$acl_all]");
  my $acl_self = $acl;
  $acl_self =~ s/^all:.*;self:(.*)$/$1/g;
  &debug("    - calculate self ACL [$acl_self]");
  if($acl_all =~ m/none/){#acl_all is none
    &debug("    - Access to 'all' granted for none");
  }else{#acl_all is none
    &debug("    - check Access ...");
    my @acl = split(' ',$acl_all);# split on space
    foreach(@acl){
      $authorization = 1 if $_ eq $role;
      $acl_type = 'all';
    }
  }
  if($acl_self =~ m/none/){#acl_self is none
    &debug("    - Access to 'self' granted for none");
  }else{#acl_self is none
    &debug("    - check Access ...");
    my @acl = split(' ',$acl_self);# split on space
    foreach(@acl){
      $authorization = 1 if $_ eq $role;
      $acl_type = 'self';
    }
  }
  if($acl_type eq "self"){
    &debug("  - authorized role [$role] by acl [$acl_self] in 'self' scope");
  }
  if($acl_type eq "all"){
    &debug("  - authorized role [$role] by acl [$acl_all] in 'all' scope");
  }

  return $acl_type, $authorization;
}

=pod

=head2 ping

The function "ping" is for testing the connection. It requires not to log
in and no arguments. It returns "OK".

=cut

sub ping {
  &debug("# sub 'ping' was called");
  return "OK";
}

=pod

=head2 sum

The function "sum" is for testing the connection. It requires not to log
in and 2 arguments. It returns the sum of the arguments as a list with 
a single line.

=cut

sub sum {
  my ($arg1, $arg2) = @_;
  &debug("# sub 'sum' was called with args: [$arg1] and [$arg2]");
  my $sum = $arg1 + $arg2;
  my @sum =();
  push(@sum,$sum);
  return [@sum];
}

# +=======================================================================+
# || login                                                               ||
# +=======================================================================+

=pod

=head2 login

my $result = login($uid,$password);

 $result = ticket, if password and login are valid
 $result = '',     if empty password
 $result = undef,  if password is wrong

=cut

sub login {
  my $uid = shift;
  my $password = shift;
  &debug("# sub 'login' was called for uid [$uid]");
  my $ticket = '';
  if(defined($password)){
    &debug("    - password was given for uid [$uid]");
    if(&check_authentication($uid,$password)){
      &debug("    - password is valid for uid [$uid]");
      # TTL = Time to live in minutes
      $time{$uid} = time()+$TTL*60;
      &debug("    - store time [$time{$uid}] for uid [$uid]");
      $ticket = &create_ticket($uid,$time{$uid});
      #&debug("  - got ticket [$ticket]");
      &debug("    - got ticket!");
    }else{
      &debug("WARN: password is valid NOT for uid [$uid]");
      $ticket = undef;
    }
  }else{
    $ticket = undef; 
    &debug("WARN: password was NOT given for uid [$uid]");
  }
  return $ticket;
}


# +=======================================================================+
# || logout                                                              ||
# +=======================================================================+

=pod

=head2 logout

my $result = logout($uid,$ticket);

 $result = true,   if uid and ticked are valid and logout successful
 $result = false,  if not

=cut

sub logout {
  &debug("# sub 'logout' [$$]");
  my $uid = shift;
  &debug("  - got uid [$uid]");

  # check if we called right with uid
  if(defined($uid)){
    # check, if the uid was authenticated before
    if(defined($time{$uid})){
      &debug("  - stored time [$time{$uid}] for uid [$uid]");
      my $ticket = shift;
      #&debug("  - ticket [$ticket]");
      &debug("  - ticket [TICKET]");
      # check, if the uid is authenticated and has a valid not expired ticket
      #&debug("call => check_ticket($uid,$ticket,$time{$uid})");
      my $status = &check_ticket($uid,$ticket,$time{$uid});

      # if ticket status OK, user is authenticated
      if($status eq 'OK'){
        &debug("  ! uid is authenticated");
        undef($time{$uid});
        return 1;
      }else{
        # uid ticket not valid
        &debug("  ! ticket not valid");
        return 0;
      }
    }else{
      # uid (never/not) logged in 
      &debug("* uid was never/not logged in!");
     return 0;
    }
  }else{
    # uid not defined
    &debug("* uid is not defined");
    return 0;
  }
  return 0;
}

# +=======================================================================+
# || session                                                             ||
# +=======================================================================+

=pod

=head2 session

my $ticket = session($uid,$ticket);

  if a ticket is returned the session is extended
  if not the session might have been expired or uid or ticket wrong


=cut

sub session {
  &debug("# sub 'session' [$$]");
  my $uid = shift;
  &debug("  - got uid [$uid]");

  # check if we called right with uid
  if(defined($uid)){
    # check, if the uid was authenticated before
    if(defined($time{$uid})){
      &debug("  - stored time [$time{$uid}] for uid [$uid]");
      my $ticket = shift;
      #&debug("  - ticket [$ticket]");
      &debug("  - ticket [TICKET]");
      # check, if the uid is authenticated and has a valid not expired ticket
      #&debug("call => check_ticket($uid,$ticket,$time{$uid})");
      my $status = &check_ticket($uid,$ticket,$time{$uid});

      # if ticket status OK, user is authenticated
      if($status eq 'OK'){
        &debug("  ! uid is authenticated");
        &debug("  - time before [$time{$uid}]");
        # TTL = Time to live in minutes
        $time{$uid} = time()+$TTL*60;
        &debug("  - store time [$time{$uid}] for uid [$uid]");
        $ticket = &create_ticket($uid,$time{$uid});
        return $ticket;
      }else{
        # uid ticket not valid
        &debug("  ! ticket not valid");
        return 0;
      }
    }else{
      # uid (never/not) logged in 
      &debug("* uid was never/not logged in!");
     return 0;
    }
  }else{
    # uid not defined
    &debug("* uid is not defined");
    return 0;
  }
  return 0;
}


# +=======================================================================+
# || ttl                                                                 ||
# +=======================================================================+

=pod

=head2 ttl

my $ttl = ttl($uid,$ticket);

  return value might be ttl or false

=cut

sub ttl {
  &debug("# sub 'ttl' [$$]");
  my $uid = shift;
  &debug("  - got uid [$uid]");

  # check if we called right with uid
  if(defined($uid)){
    # check, if the uid was authenticated before
    if(defined($time{$uid})){
      &debug("  - stored time [$time{$uid}] for uid [$uid]");
      my $ticket = shift;
      #&debug("  - ticket [$ticket]");
      &debug("  - ticket [TICKET]");
      # check, if the uid is authenticated and has a valid not expired ticket
      #&debug("call => check_ticket($uid,$ticket,$time{$uid})");
      my $status = &check_ticket($uid,$ticket,$time{$uid});

      # if ticket status OK, user is authenticated
      if($status eq 'OK'){
        &debug("  ! uid is authenticated");
        &debug("  - ttl [$TTL]");
        # TTL = Time to live in minutes
        return $TTL;
      }else{
        # uid ticket not valid
        &debug("  ! ticket not valid");
        return 0;
      }
    }else{
      # uid (never/not) logged in 
      &debug("* uid was never/not logged in!");
     return 0;
    }
  }else{
    # uid not defined
    &debug("* uid is not defined");
    return 0;
  }
  return 0;
}


=pod

=head1 PRIVATE FUNCTIONS

The function to calculate the login values should not be public. Also some 
auxiliary functions like 'debug'.

=cut

# +=======================================================================+
# || create_ticket                                                       ||
# +=======================================================================+

sub create_ticket {
  my $uid    = shift;
  my $time   = shift;
  &debug("  + sub 'create_ticket' called! [$uid] [$time]");
  my $md5    = md5_hex(join('',$time,$seed,$uid));
  #&debug("    - md5 sum: [$md5]");
  return $md5;
}

# +=======================================================================+
# || check_ticket                                                        ||
# +=======================================================================+

sub check_ticket {
  my $uid    = shift;
  my $ticket = shift;
  my $time   = shift;
  my $now    = time();
  &debug("  + sub 'check_ticket' called:");
  &debug("    - uid       = [$uid]");
  &debug("    - time      = [$time]");
  &debug("    - now       = [$now]");
  #&debug("    - ticket    = [$ticket]");
  &debug("    - ticket    = [TICKET]");
  &debug("call => create_ticket($uid,$time)");
  my $recreated = create_ticket($uid,$time);
  #&debug("    - recreated = [$recreated]");
  if($time >= $now and $ticket eq $recreated){
    return "OK";
  }elsif($time < $now){
    return "cipux-xmlrpc-d: ERROR 0003 (ticket timeout)";
  }else{
    return "cipux-xmlrpc-d: ERROR 0004 (no valid ticket)";
  }
}

# +=======================================================================+
# || check_authentication                                                ||
# +=======================================================================+
sub check_authentication {
  my $uid = shift;
  my $password = shift;
  &debug("  + sub 'check_authentication' was called for uid [$uid]");
  &debug("    - CipUX_RPC_Use_PAM  = [$cipux{CipUX_RPC_Use_PAM}]");
  &debug("    - CipUX_RPC_Use_LDAP = [$cipux{CipUX_RPC_Use_LDAP}]");
  my $authenticated = 0;
  if($cipux{CipUX_RPC_Use_PAM}){
    &debug("    - using PAM to authenticate uid");
    if(defined($password)){# just to be save
      if($password ne ''){# just to be save
	my $pam = Authen::Simple::PAM->new(service => 'sshd');
	if($pam->authenticate($uid,$password)){
	  # successfull authentication
	  &debug("    - authentication sucessfully for [$uid]!");
	  $authenticated = 1;
	}
      }
    }
  }
  if($cipux{CipUX_RPC_Use_LDAP} and not $authenticated){
    # use LDAP
    &debug("    - using LDAP to authenticate uid");
    my $encrypted = qx($GET -u $uid -e userPassword);
    chomp($encrypted);
    if(defined($encrypted)){# Bug in Authen::Simple::Password ? 
      if($encrypted ne ''){# Bug in Authen::Simple::Password ? 
	if(Authen::Simple::Password->check($password,$encrypted)){
	  &debug("    - authentication successfully for [$uid]!");
	  $authenticated = 1;
	}
      }
    }
  }
  if(not $cipux{CipUX_RPC_Use_LDAP} and not $cipux{CipUX_RPC_Use_PAM}){
    &debug("WARN: No valid login schema defined (CipUX_RPC_Use_LDAP or CipUX_RPC_Use_PAM): change /etc/cipux/rpc/cipux_rpc.conf");
  }
  return $authenticated;
}

# +=======================================================================+
# || debug                                                               ||
# +=======================================================================+
sub debug{
  if($DEBUG){
    my $date = `date`;
    my $msg = shift;
    chomp($date);
    open(LOG,">>/var/log/cipux/cipux_rpcd.log");
    print LOG "cipux_rpcd ($date): $msg\n";
    close(LOG);
  }
}

#===========================================================================
#==== Documentation ========================================================
#===========================================================================

=head1 SEE ALSO

See the CipUX webpage and the manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2006 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
